@@ -87,7 +87,17 @@ class BinaryOp : public BinaryOpShared {
 
   void Compute(OpKernelContext* ctx) override {
     const Tensor& input_0 = ctx->input(0);
+    OP_REQUIRES(ctx, input_0.dtype() == DataTypeToEnum<Tin>::v(),
+                errors::InvalidArgument(
+                    "Expected tensor of type ",
+                    DataTypeString(DataTypeToEnum<Tin>::v()), " but got type ",
+                    DataTypeString(input_0.dtype())));
     const Tensor& input_1 = ctx->input(1);
+    OP_REQUIRES(ctx, input_1.dtype() == DataTypeToEnum<Tin>::v(),
+                errors::InvalidArgument(
+                    "Expected tensor of type ",
+                    DataTypeString(DataTypeToEnum<Tin>::v()), " but got type ",
+                    DataTypeString(input_1.dtype())));
     const Device& eigen_device = ctx->eigen_device<Device>();
     bool error = false;
     bool* const error_ptr = Functor::has_errors ? &error : nullptr;
@@ -440,14 +450,6 @@ struct BinaryFunctor<CPUDevice, Functor, 2, false> {
     Assign(d, out, in.unaryExpr(Unary(scalar.data())));
   }
 
-#if !defined(EIGEN_HAS_INDEX_LIST)
-  inline Eigen::DSizes<int, 2> NByOne(int n) {
-    return Eigen::DSizes<int, 2>(n, 1);
-  }
-  inline Eigen::DSizes<int, 2> OneByM(int m) {
-    return Eigen::DSizes<int, 2>(1, m);
-  }
-#else
   inline Eigen::IndexList<int, Eigen::type2index<1>> NByOne(int n) {
     Eigen::IndexList<int, Eigen::type2index<1>> ret;
     ret.set(0, n);
@@ -458,7 +460,6 @@ struct BinaryFunctor<CPUDevice, Functor, 2, false> {
     ret.set(1, m);
     return ret;
   }
-#endif
 
   void BCast(const CPUDevice& dev,
              typename TTypes<typename Functor::out_type, NDIMS>::Tensor out,
@@ -608,6 +609,14 @@ struct UnaryFunctor<CPUDevice, Functor> {
   }
 };
 
+template <typename Functor, typename Targ>
+struct UnaryFunctorWithArg<CPUDevice, Functor, Targ> {
+  void operator()(const CPUDevice& d, typename Functor::tout_type out,
+                  typename Functor::tin_type in, Targ val) {
+    Assign(d, out, in.unaryExpr(typename Functor::func(val)));
+  }
+};
+
 // Partial specialization of ApproximateEqual<Device=CPUDevice, T>.
 template <typename T>
 struct ApproximateEqual<CPUDevice, T> {
